--
-- Filename: notesScript.lua
-- Author: Martin Fabík (LoogleCZ)
--
-- No modification without my permission!
-- Please report all bugs at mar.fabik@gmail.com
--
-- version ID   - 1.1
-- version date - 23.7.2016 (13:30)
--

Notes = {};

function Notes:loadMap(name)
	-- variables
	self.firstUpdate = true;
	self.notes = {};
	self.isServer = g_server ~= nil;
	self.isClient = g_client ~= nil;
	
	if self.isServer and self.isClient then
		self.isClient = self.isClient and not g_currentMission.connectedToDedicatedServer;
	end;
	
	self.settings = {};
	self.settings.userAddNotes = 1;
	self.settings.userDeleteNotes = 2;
	self.settings.usersEditNotes = 2;
	self.settings.userMaxNoteCount = 3;
	self.settings.afterEditNote = 1;
	
	if self.isServer then
		local savegameIndex = g_currentMission.missionInfo.savegameIndex;
		local savegameFolderPath = g_currentMission.missionInfo.savegameDirectory;
		if savegameFolderPath == nil then
			savegameFolderPath = ('%ssavegame%d'):format(getUserProfileAppPath(), savegameIndex);
		end;

		if fileExists(savegameFolderPath .. '/notes.xml') then
			local xml = loadXMLFile("notes",savegameFolderPath .. '/notes.xml');
			local name = "notes.note(%d)";
			local i = 0;
			while true do
				if not hasXMLProperty(xml, string.format(name, i)) then
					break;
				end;
				local note = {};
				note.name = getXMLString(xml, string.format(name, i) .. ".name");
				note.content = getXMLString(xml, string.format(name, i)..".content");
				note.author = getXMLString(xml, string.format(name, i)..".author");
				table.insert(self.notes, note);
				i = i + 1;
			end;
			self.settings.userAddNotes = Utils.getNoNil(getXMLInt(xml, "notes.settings#addNotes"),self.settings.userAddNotes);
			self.settings.userDeleteNotes = Utils.getNoNil(getXMLInt(xml, "notes.settings#removeNotes"),self.settings.userDeleteNotes);
			self.settings.usersEditNotes = Utils.getNoNil(getXMLInt(xml, "notes.settings#editNotes"),self.settings.usersEditNotes);
			self.settings.userMaxNoteCount = Utils.getNoNil(getXMLInt(xml, "notes.settings#maxNotesPerUser"),self.settings.userMaxNoteCount);
			self.settings.afterEditNote = Utils.getNoNil(getXMLInt(xml, "notes.settings#afterEditNote"),self.settings.afterEditNote);
			self.settings.userAddNotes = math.max(1, math.min(self.settings.userAddNotes,2));
			self.settings.userDeleteNotes = math.max(1, math.min(self.settings.userDeleteNotes,2));
			self.settings.usersEditNotes = math.max(1, math.min(self.settings.usersEditNotes,2));
			self.settings.userMaxNoteCount = math.max(1, math.min(self.settings.userMaxNoteCount,5));
			self.settings.afterEditNote = math.max(1, math.min(self.settings.afterEditNote,2));
			delete(xml);
		end;
	end;
	
	if self.isClient then
		self.status = false;
		self.noteElements = {};
		self.settingsGUIElements = {};
		self.BSIsPressed = false;
		self.timerDt = 0;
		self.firstChar = true;
		self.newNoteMode = true;
		self.listGui = RafftnixGUI:new(nil, false, false, false);
		self.removeItemGUI = RafftnixGUI:new(nil, false, false, false);
		self.NoteGUI = RafftnixGUI:new(nil, false, false, false);
		self.addEditNoteGUI = RafftnixGUI:new(nil, true, true, false);
		self.settingsGUI = RafftnixGUI:new(nil, true, true, false);
		local element = nil;

		-- listGui
		local bgl = self.listGui:createImage(self.listGui.baseElement, 0.8264, 0.335, 0.174, 0.4687, "textures/bg_white.dds");
		local bgh = self.listGui:createImage(self.listGui.baseElement, 0.8264, 0.854, 0.174, 0.0539, "textures/bg_black_alpha.dds");
		self.settingsGUIElements.openSettingsGUI = self.listGui:createImgButton(bgh, 0.145, -0.001, 0.0303, 0.054, "textures/settings.dds", function(...)
			Notes:settingsGUIOpen();
		end, self, nil);
		self.settingsGUIElements.openSettingsGUI:registerMouseOver("textures/settings_hover.dds");
		self.listGui:createText(bgh, 0, 0, 0.1733, 0.0539, g_i18n:getText("Notes_mainHeading"), 0.02, 1):setTextFocus("middle", "center");
		element = self.listGui:createImgButton(bgl, 0, 0.4688, 0.174, 0.05, "textures/arrow_up.dds", function(...)
			self.itemList:scrollUp();
		end, self, nil);
		element:registerMouseOver("textures/arrow_up_hover.dds");
		element.inactiveOverlayId = createImageOverlay(RafftnixGUIHelper.modDir .. "textures/arrow_up_inactive.dds");
		local prewB = element;
		element = self.listGui:createImgButton(bgl, 0, 0, 0.174, 0.05, "textures/arrow_down.dds", function(...)
			self.itemList:scrollDown();
		end, self, nil);
		element:registerMouseOver("textures/arrow_down_hover.dds");
		element.inactiveOverlayId = createImageOverlay(RafftnixGUIHelper.modDir .. "textures/arrow_down_inactive.dds");
		local nextB = element;
		self.itemList = self.listGui:createList(bgl, 0, 0.05, 0.174, 0.4187, (0.4187/3), 3, Notes.selectNote, self, false, {prewiousA=prewB,nextA=nextB});
		element = self.listGui:createImgButton(self.listGui.baseElement,  0.8264, 0.275, 0.174, 0.06, "textures/bg_black_alpha.dds", function(...)
			self.newNoteMode = true;
			self.editAddNoteHeading:setText(g_i18n:getText("Notes_addNote"));
			Notes:addNoteGUI();
		end, self, nil);
		element:registerMouseOver("textures/bg_black_alpha.dds");
		element:setText(g_i18n:getText("Notes_addNote"));
		element.textSize = 0.021;
		element:setTextFocus("middle","center");
		self.settingsGUIElements.addNoteButton = element;
		-- listGui end

		-- removeItemGui
		local bgc = self.removeItemGUI:createImage(self.removeItemGUI.baseElement, 0.3, 0.375, 0.4, 0.25, "textures/bg_white.dds");
		self.removeItemGUI:createText(bgc, 0, 0.07, 0.4, 0.18, g_i18n:getText("Notes_removeNotePrompt"), 0.026, {0,0,0,1}):setTextFocus("middle","center");
		element = self.removeItemGUI:createImgButton(bgc,  0, 0, 0.199, 0.07, "textures/bg_black.dds", function(...)
			if self.selectedIndex ~= nil then
				removeNoteEvent:sendEvent(self.selectedIndex);
			end;
		end , self, nil);
		element:registerMouseOver("textures/bg_blue.dds");
		element:setText(g_i18n:getText("Notes_yes"));
		element.textSize = 0.028;
		element:setTextFocus("middle","center");
		element = self.removeItemGUI:createImgButton(bgc,  0.201, 0, 0.199, 0.07, "textures/bg_black.dds", function(...)
			if self.selectedIndex ~= nil then
				Notes:selectNote(self.selectedIndex);
			end;
			self.removeItemGUI:close();
			Notes:freezePlayer();
		end , self, nil);
		element:registerMouseOver("textures/bg_blue.dds");
		element:setText(g_i18n:getText("Notes_no"));
		element.textSize = 0.028;
		element:setTextFocus("middle","center");
		-- removeItemGUI end

		-- noteGui start
		bgc = self.NoteGUI:createImage(self.NoteGUI.baseElement, 0.2, 0.28, 0.6, 0.44, "textures/bg_white.dds");
		bgh = self.NoteGUI:createImage(self.NoteGUI.baseElement, 0.2, 0.72, 0.6, 0.06, "textures/bg_black_alpha.dds");
		self.noteElements.header = self.NoteGUI:createText(bgh, 0, 0, 0.6, 0.06, "", 0.025, 1);
		self.noteElements.header:setTextFocus("middle", "center");
		self.NoteGUI:createImgButton(bgh, 0.568, 0, 0.032, 0.06, "textures/close.dds", function(...)
			self.NoteGUI:close();
			self.selectedIndex = nil;
			Notes:freezePlayer();
		end, self, nil):registerMouseOver("textures/close_hover.dds");
		self.noteElements.bodytext = self.NoteGUI:createImage(bgc, 0.008, 0.008, 0.584, 0.424, "textures/bg_transparent.dds");
		self.noteElements.author = self.NoteGUI:createText(bgc, 0.008, 0.078, 0.584, 0.025, "", 0.025, {0,0,0,1});
		element = self.NoteGUI:createImgButton(bgc,  0, 0, 0.299, 0.07, "textures/bg_black.dds", function(...)
			self.removeItemGUI:open();
			self.NoteGUI:close();
			Notes:freezePlayer();
		end , self, nil);
		element:registerMouseOver("textures/bg_blue.dds");
		element:setText(g_i18n:getText("Notes_remove"));
		element.textSize = 0.025;
		element:setTextFocus("middle","center");
		self.noteElements.btnRemove = element;
		element = self.NoteGUI:createImgButton(bgc,  0.301, 0, 0.299, 0.07, "textures/bg_black.dds", function(...)
			if self.notes[self.selectedIndex] ~= nil then
				text = self.notes[self.selectedIndex].content;
				heading = self.notes[self.selectedIndex].name;
				self.NoteGUI:close();
				Notes:freezePlayer();
				self.newNoteMode = false;
				self.editAddNoteHeading:setText(g_i18n:getText("Notes_editNote"));
				Notes:addNoteGUI();
				Notes:getTextLines(self.addEditNoteGUI,self.NewNoteTextField,0.008,0.567,0.684, 0.03,0.025,(text),15);
				Notes:getTextLines(self.addEditNoteGUI,self.NewNoteHeadingField,0.008,0.017,0.684, 0.025,0.025,(heading),1);
			end;
		end , self, nil);
		element:registerMouseOver("textures/bg_blue.dds");
		element:setText(g_i18n:getText("Notes_edit"));
		element.textSize = 0.025;
		element:setTextFocus("middle","center");
		self.noteElements.btnEdit = element;
		-- noteGui end

		-- addEditNoteGUI start
		self.addEditNoteGUI:createImage(self.addEditNoteGUI.baseElement, 0, 0, 1, 1, "textures/bg_image.dds");
		local bgh = self.addEditNoteGUI:createImage(self.addEditNoteGUI.baseElement, 0.05, 0.89, 0.9, 0.06, "textures/bg_black_alpha.dds");
		local bgc = self.addEditNoteGUI:createImage(self.addEditNoteGUI.baseElement, 0.05, 0.05, 0.9, 0.84, "textures/bg_white.dds");
		element = self.addEditNoteGUI:createText(bgh, 0, 0, 0.9, 0.06, g_i18n:getText("Notes_addNote"), 0.03, 1);
		element:setTextFocus("middle", "center");
		self.editAddNoteHeading = element;
		self.addEditNoteGUI:createImgButton(bgh, 0.868, 0, 0.032, 0.06, "textures/close.dds", function(...)
			if not self.newNoteMode and self.selectedIndex ~= nil then
				Notes:selectNote(self.selectedIndex);
			end;
			self.addEditNoteGUI:close();
			self.listGui:open();
			Notes:freezePlayer();
		end, self, nil):registerMouseOver("textures/close_hover.dds");
		element = self.listGui:createImgButton(bgc,  0, 0, 0.4495, 0.1, "textures/bg_black.dds", function(...)
			if not self.newNoteMode and self.selectedIndex ~= nil then
				Notes:selectNote(self.selectedIndex);
			end;
			self.addEditNoteGUI:close();
			self.listGui:open();
			Notes:freezePlayer();
		end, self, nil);
		element:registerMouseOver("textures/bg_blue.dds");
		element:setText(g_i18n:getText("Notes_back"));
		element.textSize = 0.03;
		element:setTextFocus("middle","center");
		element = self.listGui:createImgButton(bgc,  0.4505, 0, 0.4495, 0.1, "textures/bg_black.dds", Notes.saveEditNote , self, nil);
		element:registerMouseOver("textures/bg_blue.dds");
		element:setText(g_i18n:getText("Notes_save"));
		element.textSize = 0.03;
		element:setTextFocus("middle","center");
		local element2 = self.addEditNoteGUI:createImage(bgc, 0, 0.1, 0.2, 0.74, "textures/bg_transparent.dds");
		element = self.addEditNoteGUI:createText(element2, 0.01, 0.68, 0.9, 0.06, g_i18n:getText("Notes_noteName"), 0.025, {0,0,0,1});
		element:setTextFocus("left","center");
		element = self.addEditNoteGUI:createText(element2, 0.01, 0.62, 0.9, 0.06, g_i18n:getText("Notes_noteContent"), 0.025, {0,0,0,1});
		element:setTextFocus("left","center");
		element2 = self.addEditNoteGUI:createImage(bgc, 0.2, 0.1, 0.7, 0.74, "textures/bg_transparent.dds");
		self.NewNoteHeadingField = self.addEditNoteGUI:createImage(element2, 0, 0.68, 0.7, 0.06, "textures/bg_transparent.dds");
		self.NewNoteHeadingField:registerMouseOver("textures/bg_blue.dds");
		self.NewNoteTextField = self.addEditNoteGUI:createImage(element2, 0, 0.08, 0.7, 0.6, "textures/bg_transparent.dds");
		self.NewNoteTextField:registerMouseOver("textures/bg_blue.dds");
		-- addEditNoteGUI end
		
		-- settings GUI start
		
		self.settingsGUI:createImage(self.settingsGUI.baseElement, 0, 0, 1, 1, "textures/bg_image.dds");
		bgh = self.settingsGUI:createImage(self.settingsGUI.baseElement, 0.05, 0.89, 0.9, 0.06, "textures/bg_black_alpha.dds");
		bgc = self.settingsGUI:createImage(self.settingsGUI.baseElement, 0.05, 0.05, 0.9, 0.84, "textures/bg_white_alpha.dds");
		element = self.settingsGUI:createText(bgh, 0, 0, 0.9, 0.06, g_i18n:getText("Notes_notesName") .. " - " .. g_i18n:getText("Notes_settingsHeading"), 0.03, 1);
		element:setTextFocus("middle", "center");
		self.settingsGUIElements.heading = element;
		
		element = self.settingsGUI:createImgButton(bgc,  0, 0, 0.4495, 0.1, "textures/bg_black.dds", function(...)
			self.settingsGUI:close();
			self.listGui:open();
			if not self.newNoteMode and self.selectedIndex ~= nil then
				Notes:selectNote(self.selectedIndex);
			end;
			Notes:freezePlayer();
		end, self, nil);
		element:registerMouseOver("textures/bg_blue.dds");
		element:setText(g_i18n:getText("Notes_back"));
		element.textSize = 0.03;
		element:setTextFocus("middle","center");
		element = self.settingsGUI:createImgButton(bgc,  0.4505, 0, 0.4495, 0.1, "textures/bg_black.dds", function(...)
			local settings = {};
			settings.addNote = math.max(1, math.min(self.settingsGUIElements.userAddEditPerm.highestVisibleEntry,2));
			settings.removeNote = math.max(1, math.min(self.settingsGUIElements.userDeletePerm.highestVisibleEntry,2));
			settings.editNote = math.max(1, math.min(self.settingsGUIElements.userEditPerm.highestVisibleEntry,2));
			settings.maxItemsPerUser = math.max(1, math.min(self.settingsGUIElements.maxItemsPerUserPerm.highestVisibleEntry,5));
			settings.afterEditNote = math.max(1, math.min(self.settingsGUIElements.afterEditPerm.highestVisibleEntry,2));
			changeSettingsEvent:sendEvent(settings);
			self.settingsGUI:close();
			self.listGui:open();
			if not self.newNoteMode and self.selectedIndex ~= nil then
				Notes:selectNote(self.selectedIndex);
			end;
			Notes:freezePlayer();
		end, self, nil);
		element:registerMouseOver("textures/bg_blue.dds");
		element:setText(g_i18n:getText("Notes_saveSettings"));
		element.textSize = 0.03;
		element:setTextFocus("middle","center");
		
		local settingsPart = self.settingsGUI:createImage(bgc, 0, 0.1, 0.9, 0.72, "textures/bg_transparent.dds");
		self.settingsGUIElements.settingsPart = settingsPart;
		element = self.settingsGUI:createText(settingsPart, 0, 0.65, 0.42, 0.03, g_i18n:getText("Notes_usersAddNotes"), 0.021, {0,0,0,1});
		element:setTextFocus("right","center");
		element = self.settingsGUI:createText(settingsPart, 0, 0.60, 0.42, 0.03, g_i18n:getText("Notes_usersDeleteNotes"), 0.021, {0,0,0,1});
		element:setTextFocus("right","center");
		element = self.settingsGUI:createText(settingsPart, 0, 0.55, 0.42, 0.03, g_i18n:getText("Notes_usersEditNotes"), 0.021, {0,0,0,1});
		element:setTextFocus("right","center");
		element = self.settingsGUI:createText(settingsPart, 0, 0.50, 0.42, 0.03, g_i18n:getText("Notes_userMaxNoteCount"), 0.021, {0,0,0,1});
		element:setTextFocus("right","center");
		element = self.settingsGUI:createText(settingsPart, 0, 0.45, 0.42, 0.03, g_i18n:getText("Notes_afterEdit"), 0.021, {0,0,0,1});
		element:setTextFocus("right","center");
		
		element = self.settingsGUI:createImgButton(settingsPart,  0.45, 0.637, 0.0271, 0.0482, "textures/arrow_left.dds", function(...)
			self.settingsGUIElements.userAddEditPerm:scrollUp();
		end, self, nil);
		element:registerMouseOver("textures/arrow_left_hover.dds");
		element.inactiveOverlayId = createImageOverlay(RafftnixGUIHelper.modDir .. "textures/arrow_left_inactive.dds");
		local prewB = element;
		element = self.settingsGUI:createImgButton(settingsPart,  0.7791, 0.637, 0.0271, 0.0482, "textures/arrow_right.dds", function(...)
			self.settingsGUIElements.userAddEditPerm:scrollDown();
		end, self, nil);
		element:registerMouseOver("textures/arrow_right_hover.dds");
		element.inactiveOverlayId = createImageOverlay(RafftnixGUIHelper.modDir .. "textures/arrow_right_inactive.dds");
		local nextB = element;
		
		element = self.settingsGUI:createList(settingsPart, 0.4781, 0.637, 0.3, 0.0482, 0.0482, 1, function(...) end, self, false,{prewiousA=prewB,nextA=nextB});
		self.settingsGUIElements.userAddEditPerm = element;
		
		element = self.settingsGUI:createImage(self.settingsGUIElements.userAddEditPerm, 0, 0.15, 0.3, 0.0482, "textures/bg_white.dds");
		element = self.settingsGUI:createText(element, 0, 0, 0.3, 0.0482, g_i18n:getText("Notes_everyone"), 0.021, {0,0,0,1});
		element:setTextFocus("middle","center");
		element.isBold = true;
		
		element = self.settingsGUI:createImage(self.settingsGUIElements.userAddEditPerm, 0, 0.15, 0.3, 0.0482, "textures/bg_white.dds");
		element = self.settingsGUI:createText(element, 0, 0, 0.3, 0.0482, g_i18n:getText("Notes_administrators"), 0.021, {0,0,0,1});
		element:setTextFocus("middle","center");
		element.isBold = true;
		
		element = self.settingsGUI:createImgButton(settingsPart,  0.45, 0.587, 0.0271, 0.0482, "textures/arrow_left.dds", function(...)
			self.settingsGUIElements.userDeletePerm:scrollUp();
		end, self, nil);
		element:registerMouseOver("textures/arrow_left_hover.dds");
		element.inactiveOverlayId = createImageOverlay(RafftnixGUIHelper.modDir .. "textures/arrow_left_inactive.dds");
		local prewB = element;
		element = self.settingsGUI:createImgButton(settingsPart,  0.7791, 0.587, 0.0271, 0.0482, "textures/arrow_right.dds", function(...)
			self.settingsGUIElements.userDeletePerm:scrollDown();
		end, self, nil);
		element:registerMouseOver("textures/arrow_right_hover.dds");
		element.inactiveOverlayId = createImageOverlay(RafftnixGUIHelper.modDir .. "textures/arrow_right_inactive.dds");
		local nextB = element;
		
		element = self.settingsGUI:createList(settingsPart, 0.4781, 0.587, 0.3, 0.0482, 0.0482, 1, function(...) end, self, false,{prewiousA=prewB,nextA=nextB});
		self.settingsGUIElements.userDeletePerm = element;
		
		element = self.settingsGUI:createImage(self.settingsGUIElements.userDeletePerm, 0, 0.15, 0.3, 0.0482, "textures/bg_white.dds");
		element = self.settingsGUI:createText(element, 0, 0, 0.3, 0.0482, g_i18n:getText("Notes_everyone"), 0.021, {0,0,0,1});
		element:setTextFocus("middle","center");
		element.isBold = true;
		
		element = self.settingsGUI:createImage(self.settingsGUIElements.userDeletePerm, 0, 0.15, 0.3, 0.0482, "textures/bg_white.dds");
		element = self.settingsGUI:createText(element, 0, 0, 0.3, 0.0482, g_i18n:getText("Notes_administratorsAndOwners"), 0.021, {0,0,0,1});
		element:setTextFocus("middle","center");
		element.isBold = true;
		
		element = self.settingsGUI:createImgButton(settingsPart,  0.45, 0.537, 0.0271, 0.0482, "textures/arrow_left.dds", function(...)
			self.settingsGUIElements.userEditPerm:scrollUp();
		end, self, nil);
		element:registerMouseOver("textures/arrow_left_hover.dds");
		element.inactiveOverlayId = createImageOverlay(RafftnixGUIHelper.modDir .. "textures/arrow_left_inactive.dds");
		local prewB = element;
		element = self.settingsGUI:createImgButton(settingsPart,  0.7791, 0.537, 0.0271, 0.0482, "textures/arrow_right.dds", function(...)
			self.settingsGUIElements.userEditPerm:scrollDown();
		end, self, nil);
		element:registerMouseOver("textures/arrow_right_hover.dds");
		element.inactiveOverlayId = createImageOverlay(RafftnixGUIHelper.modDir .. "textures/arrow_right_inactive.dds");
		local nextB = element;
		
		element = self.settingsGUI:createList(settingsPart, 0.4781, 0.537, 0.3, 0.0482, 0.0482, 1, function(...) end, self, false, {prewiousA=prewB, nextA=nextB});
		self.settingsGUIElements.userEditPerm = element;
		
		element = self.settingsGUI:createImage(self.settingsGUIElements.userEditPerm, 0, 0.15, 0.3, 0.0482, "textures/bg_white.dds");
		element = self.settingsGUI:createText(element, 0, 0, 0.3, 0.0482, g_i18n:getText("Notes_everyone"), 0.021, {0,0,0,1});
		element:setTextFocus("middle","center");
		element.isBold = true;
		
		element = self.settingsGUI:createImage(self.settingsGUIElements.userEditPerm, 0, 0.15, 0.3, 0.0482, "textures/bg_white.dds");
		element = self.settingsGUI:createText(element, 0, 0, 0.3, 0.0482, g_i18n:getText("Notes_administratorsAndOwners"), 0.021, {0,0,0,1});
		element:setTextFocus("middle","center");
		element.isBold = true;
		
		element = self.settingsGUI:createImgButton(settingsPart,  0.45, 0.487, 0.0271, 0.0482, "textures/arrow_left.dds", function(...)
			self.settingsGUIElements.maxItemsPerUserPerm:scrollUp();
		end, self, nil);
		element:registerMouseOver("textures/arrow_left_hover.dds");
		element.inactiveOverlayId = createImageOverlay(RafftnixGUIHelper.modDir .. "textures/arrow_left_inactive.dds");
		local prewB = element;
		element = self.settingsGUI:createImgButton(settingsPart,  0.7791, 0.487, 0.0271, 0.0482, "textures/arrow_right.dds", function(...)
			self.settingsGUIElements.maxItemsPerUserPerm:scrollDown();
		end, self, nil);
		element:registerMouseOver("textures/arrow_right_hover.dds");
		element.inactiveOverlayId = createImageOverlay(RafftnixGUIHelper.modDir .. "textures/arrow_right_inactive.dds");
		local nextB = element;
		
		element = self.settingsGUI:createList(settingsPart, 0.4781, 0.487, 0.3, 0.0482, 0.0482, 1, function(...) end, self, false, {prewiousA=prewB, nextA=nextB});
		self.settingsGUIElements.maxItemsPerUserPerm = element;
		
		element = self.settingsGUI:createImage(self.settingsGUIElements.maxItemsPerUserPerm, 0, 0.15, 0.3, 0.0482, "textures/bg_white.dds");
		element = self.settingsGUI:createText(element, 0, 0, 0.3, 0.0482, "1", 0.021, {0,0,0,1});
		element:setTextFocus("middle","center");
		element.isBold = true;
		
		element = self.settingsGUI:createImage(self.settingsGUIElements.maxItemsPerUserPerm, 0, 0.15, 0.3, 0.0482, "textures/bg_white.dds");
		element = self.settingsGUI:createText(element, 0, 0, 0.3, 0.0482, "5", 0.021, {0,0,0,1});
		element:setTextFocus("middle","center");
		element.isBold = true;
		
		element = self.settingsGUI:createImage(self.settingsGUIElements.maxItemsPerUserPerm, 0, 0.15, 0.3, 0.0482, "textures/bg_white.dds");
		element = self.settingsGUI:createText(element, 0, 0, 0.3, 0.0482, "10", 0.021, {0,0,0,1});
		element:setTextFocus("middle","center");
		element.isBold = true;
		
		element = self.settingsGUI:createImage(self.settingsGUIElements.maxItemsPerUserPerm, 0, 0.15, 0.3, 0.0482, "textures/bg_white.dds");
		element = self.settingsGUI:createText(element, 0, 0, 0.3, 0.0482, "25", 0.021, {0,0,0,1});
		element:setTextFocus("middle","center");
		element.isBold = true;
		
		element = self.settingsGUI:createImage(self.settingsGUIElements.maxItemsPerUserPerm, 0, 0.15, 0.3, 0.0482, "textures/bg_white.dds");
		element = self.settingsGUI:createText(element, 0, 0, 0.3, 0.0482, g_i18n:getText("Notes_unlimited"), 0.021, {0,0,0,1});
		element:setTextFocus("middle","center");
		element.isBold = true;
		
		element = self.settingsGUI:createImgButton(settingsPart,  0.45, 0.437, 0.0271, 0.0482, "textures/arrow_left.dds", function(...)
			self.settingsGUIElements.afterEditPerm:scrollUp();
		end, self, nil);
		element:registerMouseOver("textures/arrow_left_hover.dds");
		element.inactiveOverlayId = createImageOverlay(RafftnixGUIHelper.modDir .. "textures/arrow_left_inactive.dds");
		local prewB = element;
		element = self.settingsGUI:createImgButton(settingsPart,  0.7791, 0.437, 0.0271, 0.0482, "textures/arrow_right.dds", function(...)
			self.settingsGUIElements.afterEditPerm:scrollDown();
		end, self, nil);
		element:registerMouseOver("textures/arrow_right_hover.dds");
		element.inactiveOverlayId = createImageOverlay(RafftnixGUIHelper.modDir .. "textures/arrow_right_inactive.dds");
		local nextB = element;
		
		element = self.settingsGUI:createList(settingsPart, 0.4781, 0.437, 0.3, 0.0482, 0.0482, 1, function(...) end, self, false, {prewiousA=prewB, nextA=nextB});
		self.settingsGUIElements.afterEditPerm = element;
				
		element = self.settingsGUI:createImage(self.settingsGUIElements.afterEditPerm, 0, 0.15, 0.3, 0.0482, "textures/bg_white.dds");
		element = self.settingsGUI:createText(element, 0, 0, 0.3, 0.0482, g_i18n:getText("Notes_remainOriginal"), 0.021, {0,0,0,1});
		element:setTextFocus("middle","center");
		element.isBold = true;

		element = self.settingsGUI:createImage(self.settingsGUIElements.afterEditPerm, 0, 0.15, 0.3, 0.0482, "textures/bg_white.dds");
		element = self.settingsGUI:createText(element, 0, 0, 0.3, 0.0482, g_i18n:getText("Notes_changeToNew"), 0.021, {0,0,0,1});
		element:setTextFocus("middle","center");
		element.isBold = true;
		
		self.settingsGUIElements.userAddEditPerm:setHighestVisibleEntry(self.settings.userAddNotes);
		self.settingsGUIElements.userDeletePerm:setHighestVisibleEntry(self.settings.userDeleteNotes);
		self.settingsGUIElements.userEditPerm:setHighestVisibleEntry(self.settings.usersEditNotes);
		self.settingsGUIElements.maxItemsPerUserPerm:setHighestVisibleEntry(self.settings.userMaxNoteCount);
		self.settingsGUIElements.afterEditPerm:setHighestVisibleEntry(self.settings.afterEditNote);
		
		-- settings GUI end

		for k,v in pairs(self.notes) do
			Notes:drawNote(v);
		end;
		self.itemList:checkScrollButtons();
		self.lastVehicleControlled = false;
	end;
	g_currentMission.NotesBase = self;
end;

function Notes:deleteMap()
end;

function Notes:saveMap()
	if g_server ~= nil then
		local savegameIndex = g_currentMission.missionInfo.savegameIndex;
		local savegameFolderPath = g_currentMission.missionInfo.savegameDirectory;
		if savegameFolderPath == nil then
			savegameFolderPath = ('%ssavegame%d'):format(getUserProfileAppPath(), savegameIndex);
		end;
		
		if fileExists(savegameFolderPath .. '/careerSavegame.xml') then
			local name = "notes";
			local xml = createXMLFile("notes", savegameFolderPath .. "/notes.xml", name);
			local j = 0;
			for k,v in pairs(g_currentMission.NotesBase.notes) do
				if v ~= nil then
					local xmlPath = string.format(name .. ".note(%d)",j);
					setXMLString(xml, xmlPath .. ".name", v.name);
					setXMLString(xml, xmlPath .. ".content", v.content);
					setXMLString(xml, xmlPath .. ".author", v.author);
					j = j + 1;
				end;
			end;
			setXMLInt(xml, "notes.settings#addNotes", g_currentMission.NotesBase.settings.userAddNotes);
			setXMLInt(xml, "notes.settings#removeNotes", g_currentMission.NotesBase.settings.userDeleteNotes);
			setXMLInt(xml, "notes.settings#editNotes", g_currentMission.NotesBase.settings.usersEditNotes);
			setXMLInt(xml, "notes.settings#maxNotesPerUser", g_currentMission.NotesBase.settings.userMaxNoteCount);
			setXMLInt(xml, "notes.settings#afterEditNote", g_currentMission.NotesBase.settings.afterEditNote);
			saveXMLFile(xml);
			delete(xml);
		end;
	end;
end;

function Notes:mouseEvent(posX, posY, isDown, isUp, button)
	if self.isClient then
		if isDown and button == 1 then
			if self.NewNoteHeadingField.mouseOver then
				self.selectedInput = self.NewNoteHeadingField;
				self.NewNoteHeadingField.isSelected = true;
				self.NewNoteTextField.isSelected = false;
			elseif self.NewNoteTextField.mouseOver then
				self.selectedInput = self.NewNoteTextField;
				self.NewNoteTextField.isSelected = true;
				self.NewNoteHeadingField.isSelected = false;
			else
				self.selectedInput = nil;
				self.NewNoteHeadingField.isSelected = false;
				self.NewNoteTextField.isSelected = false;
			end;
		end;
	end;
end;

function Notes:keyEvent(unicode, sym, modifier, isDown)
	if self.isClient then
		if sym == 8 and isDown then
			self.BSIsPressed = true;
		else
			self.BSIsPressed = false;
		end;
		self.firstChar = true;
		self.timerDt = 0;

		if self.selectedInput ~= nil then
			if self.selectedInput.visibility and isDown then
				local text = Notes:getTextContentOfTexts(self.selectedInput);
				local length = utf8Strlen(text);
				if (1300 > length and self.selectedInput == self.NewNoteTextField) or (50 > length and self.selectedInput == self.NewNoteHeadingField) then
					if unicode ~= 0 then
						self.selectedInput.elements = {};
						if self.selectedInput == self.NewNoteTextField then
							Notes:getTextLines(self.addEditNoteGUI,self.selectedInput,0.008,0.567,0.684, 0.03,0.025,(text .. unicodeToUtf8(unicode)),15);
						else
							Notes:getTextLines(self.addEditNoteGUI,self.selectedInput,0.008,0.017,0.684, 0.025,0.025,(text .. unicodeToUtf8(unicode)),1);
						end;
					end;
				end;
			end;
		end;
	end;
end;

function Notes:delete()
end;

function Notes:draw(dt)
end;

function Notes:update(dt)
	if self.isClient then
		if self.firstUpdate then
			self.firstUpdate = false;
			if g_server == nil then
				g_client:getServerConnection():sendEvent(joinClientRequestSynch:new());
			end;
		end;
		if g_currentMission.controlledVehicle == nil then
			if self.selectedInput ~= nil then
				if self.BSIsPressed then
					if (self.firstChar and self.timerDt == 0 ) or (self.timerDt/1000 > 1 and self.firstChar) or (not self.firstChar and self.timerDt/50 > 1) then
						local text = Notes:getTextContentOfTexts(self.selectedInput);
						self.selectedInput.elements = {};
						local length = utf8Strlen(text);
						if self.selectedInput == self.NewNoteTextField then
							Notes:getTextLines(self.addEditNoteGUI,self.selectedInput,0.008,0.567,0.684, 0.03,0.025,(utf8Substr(text, 0, length-1)),15);
						else
							Notes:getTextLines(self.addEditNoteGUI,self.selectedInput,0.008,0.017,0.684, 0.025,0.025,(utf8Substr(text, 0, length-1)),1);
						end;
						if self.timerDt/1000 > 1 then
							self.firstChar = false;
						end;
						self.timerDt = 0;
					end;
					self.timerDt = self.timerDt + dt;
				end;
			end;

			if self.selectedInput ~= nil then
				if self.selectedInput.mouseOverOverlayId ~= nil then
					renderOverlay(self.selectedInput.mouseOverOverlayId, x, y, self.selectedInput.width, self.selectedInput.height);
				end;
			end;

			if g_gui.currentGuiName ~= "ChatDialog" then
				if InputBinding.hasEvent(InputBinding.notesOpenClose) and not self.NoteGUI.baseElement.visibility and not self.addEditNoteGUI.baseElement.visibility and not self.removeItemGUI.baseElement.visibility then
					if self.listGui.baseElement.visibility and self.status then
						self.listGui:close();
						g_currentMission.isPlayerFrozen = false;
						InputBinding.setShowMouseCursor(false);
						g_gui.currentGuiName = "";
						self.status = false;

						for _,v in pairs(g_currentMission.controlledVehicles) do
							if v.cameras ~= nil then
								for _,v2 in pairs(v.cameras) do
									v2.isActivated = true;
								end;
							end;
						end;
					elseif self.listGui.baseElement.visibility and not self.status then
						g_gui:showGui("LoogleCZ_Notes");
						Notes:freezePlayer();
						self.status = true;
						for _,v in pairs(g_currentMission.controlledVehicles) do
							if v.cameras ~= nil then
								for _,v2 in pairs(v.cameras) do
									v2.isActivated = false;
								end;
							end;
						end;
					else
						self.listGui:open();
						self.status = false;
					end;
				end;

				if InputBinding.hasEvent(InputBinding.MENU) then
					self.listGui:close();
					self.addEditNoteGUI:close();
					self.NoteGUI:close();
					self.removeItemGUI:close();
					self.settingsGUI:close();
					self.status = false;
				end;
			end;
			self.lastVehicleControlled = false;
		end;
		if not self.lastVehicleControlled and g_currentMission.controlledVehicle ~= nil then
			self.listGui:close();
			self.addEditNoteGUI:close();
			self.NoteGUI:close();
			self.removeItemGUI:close();
			self.settingsGUI:close();
			self.status = false;
			self.lastVehicleControlled = true;
		end;
		
		if self.listGui.baseElement.visibility and self.status then
			Notes:freezePlayer();
		end;

		if self.settings.userAddNotes == 2 then
			if g_client.networkListener.isMasterUser or (self.isClient and self.isServer) then
				self.settingsGUIElements.addNoteButton.visibility = true;
			else
				self.settingsGUIElements.addNoteButton.visibility = false;
			end;
		else
			self.settingsGUIElements.addNoteButton.visibility = true;
			Notes:checkNotesCount();
		end;
		
		if g_client.networkListener.isMasterUser or (self.isClient and self.isServer) then
			self.settingsGUIElements.openSettingsGUI:setVisibility(true);
		else
			self.settingsGUIElements.openSettingsGUI:setVisibility(false);
		end;
	end;
end;

function Notes:getCharsPerLine(textContent,width,textHeight)
	local i = 1;
	local textLenght = utf8Strlen(textContent);
	while true do
		if getTextWidth(textHeight,utf8Substr(textContent, 0, i)) > width  then
			return (i-1);
		end;
		if i == textLenght then
			return (math.huge);
		end;
		i = i + 1;
	end;
end;

function Notes:getTextLines(g,b,x,y,w,h,th,textContent,maxLines,bold)
	local texts = {};
	local j = 0;
	while textContent ~= nil and utf8Strlen(textContent) > 0 do
		local chars = Notes:getCharsPerLine(textContent,w,th);
		local i = 0;
		if chars > utf8Strlen(textContent) then
			table.insert(texts,g:createText(b, x, y-(j*h), w, h, utf8Substr(textContent, 0), th,{0,0,0,1}));
			break;
		end;
		while true do
			if utf8Substr(textContent, math.min(chars - i, utf8Strlen(textContent)-1), 1) == " " then
				table.insert(texts,g:createText(b, x, y-(j*h), w, h, utf8Substr(textContent, 0, math.max(chars - i + 1,0)), th,{0,0,0,1}));
				local docasna = utf8Substr(textContent,chars-i+1);
				textContent = docasna;
				break;
			elseif (chars - i - 1) == 0 then
				table.insert(texts,g:createText(b, x, y-(j*h), w, h, utf8Substr(textContent, 0, math.max(chars - 5,0)), th,{0,0,0,1}));
				local docasna = utf8Substr(textContent,chars-5);
				textContent = docasna;
				break;
			end;
			i = i + 1;
		end;
		j = j + 1;
		if j == maxLines then
			texts[#texts]:setText(utf8Substr(texts[#texts].text,0,utf8Strlen(texts[#texts].text)-3) .. "...");
			break;
		end;
	end;
	return texts;
end;

function Notes:getTextContentOfTexts(element)
	local l = "";
	for k,v in pairs(element.elements) do
		if v.text ~= nil then
			l = l .. v.text;
		end;
	end;
	return l;
end;

function Notes:selectNote(index)
	self.noteElements.header:setText(string.format("%s",tostring(self.notes[index].name)));
	self.noteElements.author:setText(string.format(g_i18n:getText("Notes_author"), tostring(self.notes[index].author)));
	self.noteElements.bodytext.elements = {};
	Notes:getTextLines(self.NoteGUI,self.noteElements.bodytext, 0, 0.405, 0.584, 0.0252, 0.0215, self.notes[index].content,13);
	if (self.settings.userDeleteNotes == 2 and (g_currentMission.player.controllerName == self.notes[index].author or g_client.networkListener.isMasterUser or (self.isClient and self.isServer))) or self.settings.userDeleteNotes == 1 then
		self.noteElements.btnRemove:setVisibility(true);
	else
		self.noteElements.btnRemove:setVisibility(false);
	end;
	if (self.settings.usersEditNotes == 2 and (g_currentMission.player.controllerName == self.notes[index].author or g_client.networkListener.isMasterUser or (self.isClient and self.isServer))) or self.settings.usersEditNotes == 1 then
		self.noteElements.btnEdit:setVisibility(true);
	else
		self.noteElements.btnEdit:setVisibility(false);
	end;
	self.NoteGUI:open();
	self.selectedIndex = index;
end;

function Notes:addNoteGUI()
	self.listGui:close();
	self.NoteGUI:close();
	self.removeItemGUI:close();
	self.addEditNoteGUI:open();
	self.NewNoteHeadingField.elements = {};
	self.NewNoteTextField.elements = {};
end;

function Notes:settingsGUIOpen()
	self.listGui:close();
	self.NoteGUI:close();
	self.removeItemGUI:close();
	self.addEditNoteGUI:close();
	self.settingsGUI:open();
end;

function Notes:saveEditNote()
	local text = Notes:getTextContentOfTexts(self.NewNoteTextField);
	local heading = Notes:getTextContentOfTexts(self.NewNoteHeadingField);
	if self.newNoteMode then
		-- no brain in morning
		if not ((text == nil or text == "") and (heading == nil or heading == "")) then
			addNoteEvent:sendEvent(heading,text,g_currentMission.player.controllerName);
		end;
	elseif not self.newNoteMode and self.selectedIndex ~= nil then
		if not ((text == nil or text == "") and (heading == nil or heading == "")) then
			local author = g_currentMission.player.controllerName;
			if self.settings.afterEditNote == 1 then
				author = self.notes[self.selectedIndex].author;
			end;
			editNoteEvent:sendEvent(self.selectedIndex,heading,text,author);
		else
			removeNoteEvent:sendEvent(self.selectedIndex);
		end;
	end;
	self.NewNoteHeadingField.elements = {};
	self.NewNoteTextField.elements = {};
	self.addEditNoteGUI:close();
	self.listGui:open();
	Notes:freezePlayer();
end;

function Notes:freezePlayer()
	InputBinding.setShowMouseCursor(true);
	g_currentMission.isPlayerFrozen = true;
end;

function Notes:drawNote(note,itemGiven)
	local item = nil;
	if itemGiven ~= nil then
		item = itemGiven;
	else
		item = self.listGui:createImage(self.itemList, 0,0.335 ,0.1733, (0.4187/3), "textures/bg_transparent.dds");
	end;
	if item ~= nil then
		self.listGui:createImage(item, 0, 0, 0.1733, 0.002, "textures/bg_item.dds");
		local texts = Notes:getTextLines(self.listGui,item,0.008,(((0.4187/3)-0.008)-0.008*2),0.1573, 0.019,0.019,note.name,2);
		for k,v in pairs(texts) do
			v.isBold = true;
		end;
		Notes:getTextLines(self.listGui,item,0.008,(((0.4187/3)-0.008)-0.025-0.008*4),0.1573, 0.019,0.019,note.content,4);
	end;
end;

function Notes:getUserNotesCount(userName)
	local c = 0;
	for _,v in pairs(self.notes) do
		if v.author == userName then
			c = c + 1;
		end;
	end;
	return c;
end;

function Notes:checkNotesCount()
	local count = Notes:getUserNotesCount(g_currentMission.player.controllerName);
	if self.settings.userMaxNoteCount == 1 then
		if count < 1 then
			self.settingsGUIElements.addNoteButton.visibility = true;
		else
			self.settingsGUIElements.addNoteButton.visibility = false;
		end;
	elseif self.settings.userMaxNoteCount == 2 then
		if count < 5 then
			self.settingsGUIElements.addNoteButton.visibility = true;
		else
			self.settingsGUIElements.addNoteButton.visibility = false;
		end;
	elseif self.settings.userMaxNoteCount == 3 then
		if count < 10 then
			self.settingsGUIElements.addNoteButton.visibility = true;
		else
			self.settingsGUIElements.addNoteButton.visibility = false;
		end;
	elseif self.settings.userMaxNoteCount == 4 then
		if count < 25 then
			self.settingsGUIElements.addNoteButton.visibility = true;
		else
			self.settingsGUIElements.addNoteButton.visibility = false;
		end;
	else
		self.settingsGUIElements.addNoteButton.visibility = true;				
	end;
end;

addModEventListener(Notes);
g_careerScreen.saveSavegame = Utils.appendedFunction(g_careerScreen.saveSavegame, Notes.saveMap);